//
// Created by philipp on 08.05.18.
//

#ifndef PC_LOGGER_H
#define PC_LOGGER_H

#include <memory>
#include <NetworkingLib/Networking.h>
#include <NetworkingLib/ServiceClient.h>

namespace pc
{

class Logger
    : public std::enable_shared_from_this<Logger>
      , private networking::Busyable
{
private:
    struct PrivateTag
    {
    };

    struct LoggingService
    {
        using RequestMessage = std::string;
        using ResponseMessage = std::string;
    };

public:
    using Ptr = std::shared_ptr<Logger>;
    using LoggingClient = networking::service::Client<LoggingService>;

    static constexpr std::size_t PORT{10207};
    static constexpr std::size_t MAX_MESSAGE_SIZE{1024};

    static Ptr create(networking::Networking & net, const std::string & host)
    { return std::make_shared<Logger>(PrivateTag{}, net, host); }

    Logger(PrivateTag, networking::Networking & net, const std::string & host);

    void start();

    void stop();

private:
    struct AsyncState
    {
        using Ptr = std::shared_ptr<AsyncState>;

        AsyncState(Logger::Ptr self)
            : lock(*self), self(self)
        {}

        networking::BusyLock lock;
        Logger::Ptr self;
    };

    networking::Networking & net;
    std::string host;
    LoggingClient::Ptr client;
    networking::time::Timer::Ptr timer;

    void log(AsyncState::Ptr state);
};

}

#endif //PC_LOGGER_H
